// form.cpp
////////////////////////////////////////

/*
 * formf[^NX
 *  Copyright (C) 2006 肩
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <memory.h>
#include <ctype.h>
#include "slibrary.h"

#include "form.h"

////////////////////////////////////////
// tH[f[^NX
////////////////////////////////////////
CFormObj::CFormObj()
{
	m_Size = 0;
	m_Name = m_FileName = m_Value = m_Value = NULL;
	m_pNext = NULL;
};

CFormObj::~CFormObj()
{
	if(m_Name)  delete [] m_Name;
	if(m_pNext) delete m_pNext;
}

int CFormObj::Set(const char * data, const char * value, int size, const char * filename, const char * contenttype)
{
	int size_name        = strlen(data);
	int size_value       = (size <= 0) ? strlen(value) : size;
	int size_filename    = strlen(filename);
	int size_contenttype = strlen(contenttype);

	m_Name = new char [size_name+1+size_filename+1+size_contenttype+1+size_value+1];
	memcpy(m_Name, data, size_name+1);
	if(value[0]){
		m_Value = &m_Name[size_name+1];
		memcpy(&m_Name[size_name+1], value, size_value+1);
		m_Size = size_value;
	}else{
		char *ptr = strchr(m_Name, '=');
		if(ptr){
			*ptr = '\0';
			m_Value = ptr+1;
			m_Size = strlen(m_Value);
		}
	}
	if(filename[0]){
		m_FileName = &m_Name[size_name+1+size_value+1];
		memcpy(&m_Name[size_name+1+size_value+1], filename, size_filename+1);
	}
	if(contenttype[0]){
		m_Type = &m_Name[size_name+1+size_value+1+size_filename+1];
		memcpy(&m_Name[size_name+1+size_value+1+size_filename+1], contenttype, size_contenttype+1);
	}
	return 0;
}

CFormObj * CFormObj::Search(const char * name)
{
	if(!strcmp(name, m_Name))
		return this;
	return (m_pNext) ? m_pNext->Search(name) : NULL;
}

#ifdef DEBUG
void CFormObj::printall(FILE *fp, int all)
{
	fputs("<p>--------------<br><tt>name: ", fp);
	fputs(m_Name, fp);

	fputs("<br>FileName:\n", fp);
	if(m_FileName) fputs(m_FileName, fp);

	fputs("<br>FileType:\n", fp);
	if(m_Type) fputs(m_Type, fp);

	fprintf(fp, "<br>value: [%d]<pre>", m_Size );
	if(all && m_Value) fputs(m_Value, fp);

	fputs("</pre></tt></p>\n", fp);

	if(m_pNext) m_pNext->printall(fp);
};
#endif

// ---------------------------------------------------
CForm::CForm()
{
	m_pTop  = NULL;
	m_pLast = &m_pTop;
	m_Sep   = NULL; // for multipart data
}

CForm::~CForm()
{
	if(m_pTop) delete m_pTop;
	if(m_Sep)  delete [] m_Sep;
}

int CForm::Init(int type, const long int max_mem)
{
	int res = 0;
	char *env = getenv("REQUEST_METHOD");

	if((type & GET) && ((type & FORCE) || (env && !strcmp(env, "GET")))){
		if(getenv("QUERY_STRING")){
			int length = strlen(getenv("QUERY_STRING"));
			char *buf = new char [length+1];
			memcpy(buf, getenv("QUERY_STRING"), length);
			buf[length] = '\0';

			Get(buf);

			delete [] buf;
		}
	}

	if((type & POST) && ((type & FORCE) || (env && !strcmp(env, "POST")))){
		char *contenttype =    getenv("CONTENT_TYPE");
		if(!contenttype)     return 1;

		long int length = atol(getenv("CONTENT_LENGTH"));
		if(length > max_mem) return 2;

		if( strcmp(contenttype, "application/x-www-form-urlencoded")
		&&  strcmp(contenttype, "application/sgml-form-urlencoded")
		&& strncmp(contenttype, "multipart/form-data;", 20))
			return 3; // unknown type

		char *buf = new char [length+1];
		if(!buf)
			return 4;

		int n = fread(buf, 1, length, stdin);
		buf[n] = '\0';

		if(!strcmp(contenttype, "application/x-www-form-urlencoded"))
			res = PostNormal(buf, "&");
		else if(!strcmp(contenttype, "application/sgml-form-urlencoded"))
			res = PostNormal(buf, ";");
		else //if(!strncmp(contenttype, "multipart/form-data;", 20))
			res = PostMultiPart(buf, n);

		if(res)
			res += 4;
		delete [] buf;
	}

	return res;
}
int CForm::URLDecode(char *write, const char *read)
{
	if(!write || !read)
		return 1;

	int a, b;
	while(*read){
		if(*read == '%'){
			if(!*++read) return 2; else a = *read;
			if(!*++read) return 2; else b = *read;
			if(!isxdigit(a) || !isxdigit(b))
				return 2;
			a = ('0' <= a && a <= '9') ? a - '0' : 10 + (('A' <= a && a <= 'F') ? a - 'A' : a - 'a');
			b = ('0' <= b && b <= '9') ? b - '0' : 10 + (('A' <= b && b <= 'F') ? b - 'A' : b - 'a');
			*write++ = a << 4 | b;

			++read;
		}else if(*read == '+'){
			*write++ = ' ';
			++read;
		}else
			*write++ = *read++;
	}
	*write = *read;

	return 0;
}

int CForm::Get(char *buf)
{
	return PostNormal(buf, "&");
}

int CForm::PostNormal(char * buf, const char *sep)
{
	char *token = strtok(buf, sep);
	while(token != NULL){
		// read
		*m_pLast = new CFormObj;
		(*m_pLast)->Set(token);

		// URL decode
		URLDecode((*m_pLast)->m_Name, (*m_pLast)->m_Name);
		if((*m_pLast)->m_Value)
			URLDecode((*m_pLast)->m_Value, (*m_pLast)->m_Value);

		// next
		m_pLast = &((*m_pLast)->m_pNext);
		token = strtok(NULL, sep);
	}
	return 0;
}

// _ӎt@C̎o
// ĂCIE͉ŃpXƓ]́H
// t@C]Ώ\Ȃ
char * CForm::SepCheck(char *filename)
{
	char *last = filename;
	char *tmp = filename;
	do{
		tmp = strchr(tmp,  '\\');
		if(!tmp)
			break;

		// ŏʃrbgĂȂ (߂ǂ̂ŁCŏʃrbĝ݂Ŕ)
		if(!(*(tmp-1) & 0x80))
			last = tmp+1;
		++tmp;
	}while(1);

	return last;
}

int CForm::PostMultiPart(char *buf, int length)
{
	char * tmp = getenv("CONTENT_TYPE");
	tmp = strstr(tmp, "boundary=");
	if(!tmp)
		return 1;
	tmp = &tmp[9];

	// boundary
	int len = strlen(tmp); // CRLF -- tmp CRLF \0
	if(m_Sep) delete [] m_Sep;
	m_Sep = new char [len+7];
	m_Sep[    0] = '\r';
	m_Sep[    1] = '\n';
	m_Sep[    2] = '-';
	m_Sep[    3] = '-';
	memcpy(&m_Sep[4], tmp, len);
	m_Sep[len+4] = '\r';
	m_Sep[len+5] = '\n';
	m_Sep[len+6] = '\0';

	// f[^擪̃`FbN
	if(strncmp(buf, &m_Sep[2], len+4))
		return 2;

	// f[^̏I[
	tmp = &buf[length];

	// p[c
	char *start, *end;
	char *name, *value;
	char *filename, *contenttype;
	int left;
	start = &buf[len+4];
	do{
		// Tׂc̈
		left = tmp - start;

		// T
		end = (char*)memstr(start, m_Sep, len+6, left);
		if(!end)
			break;
		*end = '\0';

		// ǂݏof[^𕪉Cf[^o^
		parse(start, name, value, filename, contenttype);
		*m_pLast = new CFormObj;
		(*m_pLast)->Set(name, value, end-value, filename, contenttype);
		m_pLast = &((*m_pLast)->m_pNext);

		// boundaryi߂
		start = end+len+6;
	}while(start < tmp);

	// Ō̕
	m_Sep[len+4] = '-';
	m_Sep[len+5] = '-';
	end = (char*)memstr(start, m_Sep, len+6, left);
	m_Sep[len+4] = '\0';
	if(!end)
		return 3;
	*end = '\0';

	// ǂݏof[^𕪉Cf[^o^
	parse(start, name, value, filename, contenttype);
	*m_pLast = new CFormObj;
	(*m_pLast)->Set(name, value, end-value, filename, contenttype);
	m_pLast = &((*m_pLast)->m_pNext);

	return 0;
}

int CForm::parse(char *str, char *&name, char *&value, char *&filename, char *&contenttype)
{
	name = value = filename = contenttype = "";

	do{
		// Content-Disposition: form-data; name="xxx" [filename="xxx"]

		if(!strncmp(str, "Content-Disposition: form-data; ", 32)){
			str += 32;

			// name
			while(*str == ' ') ++str;
			name = str + 6;
			str = strchr(name, '"');
			*str = '\0';
			++str;

			if(*str == ';'){
				++str;

				// filename
				while(*str == ' ') ++str;
				str += 10;
				filename = str;

				// I[L̒T
				while(*str != '\"'){
					if(*str == '\\')
						++str;
					++str;
				}
				*str = '\0';

				// Zp[^`FbN
//printf("path: %s<br>\n", filename);

				filename = SepCheck(filename);

				++str;
			}
		}
		// Content-Type: xxx
		else if(!strncmp(str, "Content-Type: ", 14)){
			str += 14;
			contenttype = str;
			str = strstr(str, "\r\n");
			*str = '\0';
		}
		// unknown xxx:
		else{
			// 1sXLbv
			str = strstr(str, "\r\n");
		}

		// \r\n
		str += 2;
	}while(str[0] != '\r' || str[1] != '\n');

	// \r\n
	str += 2;

	value = str;
	return 0;
}


#ifdef DEBUG
void CForm::printall(FILE *fp, int all){
	if(m_pTop) m_pTop->printall(fp, all);
}
#endif
