// auth.cpp

/* Authentication plugin 
 *  Copyright (C) Serika-Shiki http://www.kis-lab.com/serikashiki/
 */

#define PLUGIN_NAME "auth.so"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/file.h>

#include "common.h"

#if defined (__cplusplus)
extern "C" {
#endif

CSString cookiedata;

int IsAuthenticated(void);

int IsEnable(void)
{	return 1; }

int Type(void)
{
	return CPlugin::EXT | CPlugin::AUTH;
}

const char *Info(void)
{	return sysmsg[PLUGIN_NAME]["info"]; }

const char *Name(void)
{	return sysmsg[PLUGIN_NAME]["name"]; }

const char *Title(void)
{	return IsAuthenticated() ? sysmsg[PLUGIN_NAME]["logout"] : sysmsg[PLUGIN_NAME]["login"]; }

int TestUser(const char *ID, const char *PW)
{
	return strcmp(ID, PW);
}

int IsAuthenticated(void)
{
//	REMOTE_ADDR
	// 1

//	GetHostByAddr
	// 2

	// form
	do{
		CFormObj *obj;
		const char *id, *pw;
		obj = form.Search("action");
		if(!obj || strcmp(obj->Value(), "login"))
			break;
		obj = form.Search("id");
		if(!obj) break;
		id = obj->Value();

		obj = form.Search("pw");
		if(!obj) break;
		pw = obj->Value();

		if(!TestUser(id, pw))
			return 3;
	}while(0);

	// cookie
	if(cookie.IsExist(PLUGIN_NAME) && !strcmp(cookie[PLUGIN_NAME] , "accept"))
		return 4;

	return 0; // Invalid user
}

int Run(void)
{
	CFormObj *obj;
	const char *id, *pw;

	do{
		obj = form.Search("action");
		if(!obj)
			break;
		if(!strcmp(obj->Value(), "login")){
			obj = form.Search("id");
			if(!obj) break;
			id = obj->Value();

			obj = form.Search("pw");
			if(!obj) break;
			pw = obj->Value();

			if(TestUser(id, pw)){
				message += sysmsg[PLUGIN_NAME]["error"];
				message += "<br>\n";
				break;
			}

			message += sysmsg[PLUGIN_NAME]["sccuess"];
			message += "<br>\n";

			cookiedata.Format("document.cookie = \"%s=accept; expires=Sat, 1-Jan-2030 00:00:00 GMT; domain=%s; path=%s;\"",
				PLUGIN_NAME, (string)env["SERVER_NAME"], (string)env["album"]);
		}else if(!strcmp(obj->Value(), "logout")){
			cookiedata.Format("document.cookie = \"%s=deny; expires=Sat, 1-Jan-2000 00:00:00 GMT; domain=%s; path=%s;\"",
				PLUGIN_NAME, (string)env["SERVER_NAME"], (string)env["album"]);
		}
	}while(0);

	return 0;
}

int Extension(void)
{
	if(cookiedata.Length()){
		puts("<script type=\"text/javascript\">\n<!--\n");
		puts(cookiedata);
		puts("// -->\n</script>\n");
//		puts(cookiedata);
	}

	if(!IsAuthenticated()){
		printf("<h5>%s</h5><div><p>", (string)sysmsg[PLUGIN_NAME]["login"]);
		puts  (sysmsg[PLUGIN_NAME]["message"]);
		printf("<form action=\"%s%s/%s%s\" method=\"post\">\n", (string)env["album"], (string)env["script"], (string)env["vhome"], (string)env["gallery"]);
		puts  ("<input type=\"hidden\" name=\"plugin\" value=\"" PLUGIN_NAME "\">\n"
			   "<input type=\"hidden\" name=\"action\" value=\"login\"><br>"
			   "<tt>ID: <input type=\"text\" name=\"id\"></tt><br>"
			   "<tt>PW: <input type=\"text\" name=\"pw\"></tt><br>"
			   "<input type=\"submit\" value=\"login\">"
		);
		puts("</form>");
	}else{
		printf("<h5>%s</h5><div><p>", (string)sysmsg[PLUGIN_NAME]["logout"]);
		printf("<form action=\"%s%s/%s%s\" method=\"post\">\n", (string)env["album"], (string)env["script"], (string)env["vhome"], (string)env["gallery"]);
		puts  ("<input type=\"hidden\" name=\"plugin\" value=\"" PLUGIN_NAME "\">\n"
			   "<input type=\"hidden\" name=\"action\" value=\"logout\"><br>"
			   "<input type=\"submit\" value=\"logout\">"
		);
		puts("</form>");
	}

	puts("<div align=\"right\"><p>Copyright (C) 2006 Serika-Shiki Authentication Plugin</p></div>");

	return 0;
}

#if defined (__cplusplus)
}
#endif
