// axzip.cpp

/* Zip Archive extract plugin */
/* powered by zziplib         */

#define PLUGIN_NAME "axzip.so"

#include <stdio.h>
#include <unistd.h>

#include <zzip.h>

#include "common.h"

#if defined(__cplusplus)
extern "C" {
#endif

ZZIP_DIR *dir = NULL;
CSString filepath;

int IsEnable(void)
{	return 1; }

int Type(void)
{	return CPlugin::AX; }

const char *Info(void)
{	return "Zip Archive extract plugin powered by zziplib"; }

const char *Name(void)
{	return "axzip"; }

int IsAccept(int fd, const void *header)
{
	return (memcmp(header, "PK\3\4", 4) == 0);
}

int OpenArchive(const char *path)
{
	// invalid path
	if(!path || !*path)
		return -1;

	// same archive
	if(dir && filepath == path)
		return 0;

	if(dir)
		zzip_dir_close(dir);
	dir = zzip_dir_open(path, NULL);
	if(!dir) // file is not zip archive
		return -1;
	filepath = path;

	return 1;
}

CFileList *FileList(const char *path)
{
	static CFileList *filelist = NULL;

	if(!path || !*path)
		return NULL;

	int res;
	res = OpenArchive(path);
	if(res == -1) return NULL;
	if(res ==  0) return filelist;

	filelist = new CFileList;
	if(!filelist)
		return NULL;
	CFileList &list = *filelist;

	ZZIP_DIRENT dirent;
	CSString tmp;
	for(int i = 0; zzip_dir_read(dir, &dirent); ++i){
		if(dirent.st_size > 0){
			list[dirent.d_name]["name"] = dirent.d_name;
			tmp.Format("%d", dirent.st_size);
			list[dirent.d_name]["size"] = tmp;
		}
	}

	return filelist;
}

int Extract(const char *path, const char *name, int fd)
{
	if(!path || !*path || !name || !*name)
		return -1;

	int res;
	res = OpenArchive(path);
	if(res == -1)
		return -1;

	ZZIP_FILE *fp;
	fp = zzip_file_open(dir, name, 0);
	if(!fp)
		return -2;

	char buf[4096];
	int len;
	do{
		len = zzip_file_read(fp, buf, sizeof(buf));
		if(len <= 0)
			break;
		res = write(fd, buf, len);
	}while(res != -1);

	zzip_file_close(fp);

	return 0;
}

#if defined(__cplusplus)
}
#endif
