// ifjpeg.cpp

/* JPEG plugin
 * powered by GD Library
 * if defined USE_EXIF, this plugin use libexif.
 */

#include <stdio.h>
#include <string.h>
#include <gd.h>

#include "common.h"

#ifdef USE_EXIF
#include <libexif/exif-data.h>
#include <libexif/exif-utils.h>
#include <libexif/exif-loader.h>
#endif

#if defined(__cplusplus)
extern "C" {
#endif

int IsEnable(void)
{	return 1; }

int Type(void)
{	return CPlugin::IF; }

char *Info(void)
{
	return  "Jpeg plugin: powered by GD Library"
#ifdef USE_EXIF
			" and libexif"
#endif
			"."; }

char *Name(void)
{	return "ifjpeg"; }

int IsAccept(const char *path, const void *header)
{
	const unsigned char *ptr = (const unsigned char *)header;
	if((ptr[0] == 0xff && ptr[1] == 0xd8))
		return 1;
	return 0;
}

size ImageSize(const char *img, const void *header)
{
	FILE *fp = fopen(img, "r");
	size imagesize;
	gdImagePtr im = NULL;

	if((im = gdImageCreateFromJpeg(fp)) == NULL)
		;
	else{
		imagesize.x = gdImageSX(im);
		imagesize.y = gdImageSY(im);
	}

	if(fp) fclose(fp);
	if(im) gdImageDestroy(im);

	return imagesize;
}

#ifdef USE_EXIF
int IsExistThumbnail(const char *in, const char *out)
{
	int res = 0;

	ExifData *ed;
	ExifLoader *l;

	l = exif_loader_new ();
//	exif_loader_log (l, log);
	exif_loader_write_file (l, in);
	ed = exif_loader_get_data (l);
	exif_loader_unref (l);

	do{
		// no thumbnail
		if (!ed || !ed->data)
			break;

		CSString tmp(out);
		tmp += ".tmp";
		FILE *fp = fopen (tmp, "wb");
		if(!fp)
			break;

		fwrite (ed->data, 1, ed->size, fp);
		fclose (fp);
		res = 1;

		message << "Load thumbnail<br>\n";
	}while(0);

	return res;
}
#endif

size MakeThumbnail(const char *in, const char *out, int width, int height, int force)
{
	CSString tmp;

#ifdef USE_EXIF
	if(IsExistThumbnail(in, out)){
		tmp = out;
		tmp += ".tmp";
		in = tmp.GetBuffer();
	}
#endif

	FILE *fp = fopen(in, "r");
	FILE *tp = fopen(out, "w");
	gdImagePtr image = NULL, thumbnail = NULL;
	int x, y;
	size imagesize;

	do{
		if(!fp || !tp)
			break;

		// load
		image = gdImageCreateFromJpeg(fp);
		if(!image)
			break;
		x = gdImageSX(image);
		y = gdImageSY(image);

		// size
		if(!force){
			if(x > width || y > height){
				if(x > y)
					height = (int)((double)height * y / x);
				else
					width  = (int)((double)width * x / y);
			}
			if(x < width && y < height){
				width = x;
				height = y;
			}
		}

		// create thumbnail
		// BGD_DECLARE(gdImagePtr) gdImageCreateTrueColor (int sx, int sy);
		thumbnail = gdImageCreateTrueColor(width, height);
		if(!thumbnail)
			break;

		// resample
		gdImageCopyResampled(thumbnail, image, 0, 0, 0, 0, width, height, x, y);
		/* BGD_DECLARE(void) gdImageCopyResampled (gdImagePtr dst, gdImagePtr src, int dstX,
                 int dstY, int srcX, int srcY, int dstW, int dstH,
                 int srcW, int srcH); */

		// save
		gdImageJpeg(thumbnail, tp, 100);
		fflush(tp);

		// print status
//		buf.Message("size: (%dx%d) -&gt; (%dx%d)<br>\n", x, y, width, height);

		// success
		imagesize.x = width;
		imagesize.y = height;
	}while(0);

	if(fp) fclose(fp);
	if(tp) fclose(tp);

	if(thumbnail) gdImageDestroy(thumbnail);
	if(image)     gdImageDestroy(image);

	return imagesize;
}

#if defined(__cplusplus)
}
#endif
