// ifpng.cpp

/* PNG plugin
 * powered by GD Library.
 */

#include <stdio.h>
#include <string.h>
#include <gd.h>

#include "common.h"

#if defined(__cplusplus)
extern "C" {
#endif

int IsEnable(void)
{	return 1; }

int Type(void)
{	return CPlugin::IF; }

char *Info(void)
{	return  "PNG plugin: powered by GD Library."; }

char *Name(void)
{	return "ifpng"; }

int IsAccept(const char *path, const void *header)
{
	if((memcmp(header, "\x89PNG\r\n\x1a\x0a", 8) == 0))
		return 1;
	return 0;
}

size ImageSize(const char *img, const void *header)
{
	size imagesize;

	const unsigned char *buf = (const unsigned char*)header;
	if((memcmp(buf, "\x89PNG\r\n\x1a\x0a", 8) == 0)){
		imagesize.x = buf[16] << 24 | buf[17] << 16 | buf[18] << 8 | buf[19];
		imagesize.y = buf[20] << 24 | buf[21] << 16 | buf[22] << 8 | buf[23];
	}

	return imagesize;
}

size MakeThumbnail(const char *in, const char *out, int width, int height, int force, const char *path)
{
	FILE *fp = fopen(in, "r");
	FILE *tp = fopen(out, "w");
	gdImagePtr image = NULL, thumbnail = NULL;
	int x, y;
	size imagesize;

	do{
        if(!fp || !tp)
            break;

		// load
		image = gdImageCreateFromPng(fp);
		if(!image)
			break;

		x = gdImageSX(image);
		y = gdImageSY(image);

		// size
		if(!force){
			if(x > width || y > height){
				if(x > y)
					height = (int)((double)height * y / x);
				else
					width  = (int)((double)width * x / y);
			}
		}

		// create thumbnail
		// BGD_DECLARE(gdImagePtr) gdImageCreateTrueColor (int sx, int sy);
		thumbnail = gdImageCreateTrueColor(width, height);
		if(!thumbnail)
			break;

		// resample
		gdImageCopyResampled(thumbnail, image, 0, 0, 0, 0, width, height, x, y);
		/* BGD_DECLARE(void) gdImageCopyResampled (gdImagePtr dst, gdImagePtr src, int dstX,
                 int dstY, int srcX, int srcY, int dstW, int dstH,
                 int srcW, int srcH); */

		// save
		gdImageJpeg(thumbnail, tp, 100);
		fflush(tp);

		// print status
//		buf.Message("size: (%dx%d) -&gt; (%dx%d)<br>\n", x, y, width, height);

		// success
		imagesize.x = width;
		imagesize.y = height;
	}while(0);

	if(fp) fclose(fp);
	if(tp) fclose(tp);

	if(thumbnail) gdImageDestroy(thumbnail);
	if(image)     gdImageDestroy(image);

	return imagesize;
}

#if defined(__cplusplus)
}
#endif
