// upload.cpp

/* File Upload Plugin */

#define PLUGIN_NAME "upload.so"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/file.h>

#include "common.h"

#if defined (__cplusplus)
extern "C" {
#endif

int IsEnable(void)
{
	if(!strcmp((string)env["script"], "gallery") && (!*(string)env["gallery"] || *(string)env["gallery"] == '.'))
		return 0;
	return 1;
}

int Type(void)
{	return CPlugin::PEXT | CPlugin::EXT; }

const char *Info(void)
{	return sysmsg[PLUGIN_NAME]["info"]; }

const char *Name(void)
{	return sysmsg[PLUGIN_NAME]["title"]; }

const char *Title(void)
{	return sysmsg[PLUGIN_NAME]["title"]; }

int Run(void)
{
	CFormObj *obj;
	int fd = -1;
	CSString realname;

	do{
		obj = form.Search("action");
		if(!obj || strcmp(obj->Value(), "upload") || strcmp("POST", env["REQUEST_METHOD"]))
			break;

		CSString folder;
//		folder = env["home"];
		if(env.IsExist("folder")){
			folder += env["folder"];
		}else{
			obj = form.Search("folder");
			if(obj && obj->Value() && *obj->Value())
				folder += obj->Value();
			else
				folder += env["gallery"];
		}

		obj = form.Search("ulfile");
		if(!obj)
			break;

		char *ptr;
		ptr = strrchr(obj->FileName(), '.');
		if(!ptr)
			break;
		if(!ini["upload"].IsExist("ext", ++ptr, 0)){
			message << sysmsg["upload"]["ext error"] << obj->FileName() << "<br>\n";
			break;
		}

		CSString hash = hashFromBuffer(obj->Value(), obj->Size());
		env["upload_hash"] = hash;

		CSString path = hashSearch(hash, "image");
		if(path.Length()){
			message << sysmsg["upload"]["file exist"] << "<br>\n";

			message << obj->FileName() << ": <a href=\"" << env["album"] << env["home"] << path << "\">";
			message << path;
			message << "</a>" << "<br>\n";

			env["upload_filename"] = path;
			break;
		}

		fd = CreateFile(folder, obj->FileName(), realname);
		if(fd == -1) break;
		env["upload_filename"] = realname;

		{
			CSString xxx;
			xxx.Format("%d", obj->Size());
			env["upload_filesize"] = xxx;
		}

		write(fd, obj->Value(), obj->Size());
		fchmod(fd, 0666);
		fsync(fd);
		env["upload"] = "1";

		if(ini["upload"]["extract archive"].IsTrue() && form.Search("ex") && ExtractArchive(realname) > 0){
			if(!form.Search("ex2"))
				ini["upload"]["extract archive"] = "0";
			unlink(realname);
		}else{
			hashRegist(hash, "image", realname);
			AddRecent(hash);
		}
	}while(0);

	if(fd != -1)
		close(fd);

	return 0;
}

int leaf(CDirTree &tree, CSString &node)
{
#ifdef DEBUG
	static int loop = 0;
	if(++loop > 100)
		return 0;
#endif

	CSString nextnode, nextnode_t;
	if(node.Length())
		nextnode_t = node + "/";

	CSString buf;
	for(int i = 0; tree[node]["entries"][i]; ++i){
		if(tree[node]["entries"][i][0] == '.')
			continue;

		nextnode  = nextnode_t;
		nextnode += tree[node]["entries"][i];

		if(!*(tree[nextnode]["name"]))
			tree[nextnode]["name"] = tree[node]["entries"][i];
		printf("<option value=\"%s\">%s(%s)</option>\n", (string)nextnode, (string)tree[nextnode]["name"], (string)nextnode);

		if(tree[nextnode]["entries"][0])
			leaf(tree, nextnode);
	}

	return 0;
}

int ShowForm(void)
{
	CSString plugin = PLUGIN_NAME;
	if(env.IsExist("parent"))
		plugin = env["parent"];


	printf( "<form action=\"%s%s/%s%s\" method=\"post\" enctype=\"multipart/form-data\">\n", (string)env["album"], (string)env["script"], (string)env["vhome"], (string)env["gallery"]);
	printf( "<input type=\"hidden\" name=\"plugin\" value=\"%s\">\n"
			"<input type=\"hidden\" name=\"action\" value=\"upload\">\n"
			, (string)plugin);

	// upload folder
	if(env["upload_selectfolder"].IsTrue()){
		puts((string)sysmsg[PLUGIN_NAME]["folder"]);
		puts("<select name=\"folder\">");
		CSString dummy;
		leaf(tree, dummy);
		puts("</select><br>");
	}

	// upload file
	printf( "<input type=\"file\" name=\"ulfile\" size=\"60\">");

	// submit
	printf( "<input type=\"submit\" value=\"%s\"><br>\n", (string)sysmsg[PLUGIN_NAME]["upload"]);

	// archive flag
	if(ini["upload"]["extract archive"].IsTrue())
		printf( "<input type=\"checkbox\" name=\"ex\" checked>%s <input type=\"checkbox\" name=\"ex2\" checked>%s ", (string)sysmsg[PLUGIN_NAME]["ex"], (string)sysmsg[PLUGIN_NAME]["ex2"]);

	puts  ( "</form>");

	// information
	puts  ( "<p>");
		if(env["upload_enableext"].IsTrue()){
			puts  ( (string)sysmsg[PLUGIN_NAME]["uploadable"]);
			puts  ( (string)ini["upload"]["ext"]);
			puts  ( "<br>");
		}

		if(env["upload_maxfilesize"].IsTrue()){
			int unit = 0;
			double size = atof(ini["upload"]["max_file_size"]);
			if(size > 1024) size /= 1024, ++unit; // K
			if(size > 1024) size /= 1024, ++unit; // M
			if(size > 1024) size /= 1024, ++unit; // G
			printf("%s%.1lf%c byte<br>\n", (string)sysmsg[PLUGIN_NAME]["max size"], size, " KMG???"[unit]);
		}
	puts  ( "</p>");

	// copyright
	if(env["upload_copy"].IsTrue()){
		puts  ( "<div align=\"right\"><p>Copyright (C) 2006 Serika-Shiki Upload Plugin</p></div>");
	}

	return 0;
}

int PageExtension(void)
{
	env["upload_selectfolder"] = "0";
	env["upload_enableext"] = "1";
	env["upload_maxfilesize"] = "1";
	env["upload_copy"] = "1";

	ShowForm();

	return 0;
}

int Extension(void)
{
	printf("<h5>%s</h5><div>\n", (string)sysmsg[PLUGIN_NAME]["title"]);

	env["upload_selectfolder"] = "1";
	env["upload_enableext"] = "1";
	env["upload_maxfilesize"] = "1";
	env["upload_copy"] = "1";

	ShowForm();

	puts  ("</div>"); // <h5>title</h5>

	return 0;
}

#if defined (__cplusplus)
}
#endif
