// move.cpp

/* move files */

#define PLUGIN_NAME "z_move.so"

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>

#include "common.h"

#if defined (__cplusplus)
extern "C" {
#endif

int IsEnable(void)
{   return 1; }

int Type(void)
{   return CPlugin::PV; }

const char *Info(void)
{   return sysmsg[PLUGIN_NAME]["info"]; }

const char *Name(void)
{   return sysmsg[PLUGIN_NAME]["title"]; }

int Move(CSString file, CSString to, CSString hash)
{
	CSString realname;

	{// create new file
		CSString basename, opt, ext;
		SplitFilename(file, basename, opt, ext);

		int fd = CreateFile(to, basename + "." + ext, realname);
		if(!fd){
			message << sysmsg[PLUGIN_NAME]["error"] << "<br>\n";
			message << to << "/" << basename << "." << ext << "<br>\n";
			return 0;
		}
		close(fd);
	}

	CSString oldpath, newpath;
	{// move image file
		oldpath  = env["home"];
		oldpath << env["gallery"] << "/" << file;
		newpath  = env["home"];
		newpath << realname;

		message << "<tt>[" << oldpath << "] to [" << newpath << "]</tt><br>\n";
		rename(oldpath, newpath);

		// hash
		hashRegist(hash, "image", realname);
	}

	CSString path, filename, ext;
	{// move other files
		SplitPath(newpath, path, filename, ext);
		oldpath = env["home"];
		oldpath << env["gallery"] << "/" << ini[""]["thumb"] << "/"; // << file << ".jpg";
		newpath = path;
		newpath << "/" << ini[""]["thumb"] << "/"; // << opt << "." << ext << ".jpg";

		do{
			DIR *dir = opendir(oldpath);
			if(!dir) break;

			CSString oldfile, newfile;
			int len = strlen(file);
			for(struct dirent *ent = readdir(dir); ent; ent = readdir(dir)){
				if(strncmp(file, ent->d_name, len))
					continue;

				oldfile  = oldpath;
				oldfile << ent->d_name;
				newfile  = newpath;
				newfile << filename << "." << ext;
				SplitPath(ent->d_name, path, filename, ext);
				newfile << "." << ext;
				message << "<tt>[" << oldfile << "] to [" << newfile << "]</tt>" << "<br>\n";
				rename(oldfile, newfile);
			}
			closedir(dir);
		}while(0);
	}

	return 0;
}

int Remove(CSString file, CSString hash)
{
	CSString oldpath;
	{// move image file
		oldpath  = env["home"];
		oldpath << env["gallery"] << "/" << file;

		message << "Remove <tt>" << oldpath << "</tt><br>\n";
		unlink(oldpath);

		// hash
		hashClear(hash);
	}

	CSString path, filename, ext;
	{// remove other files
		oldpath = env["home"];
		oldpath << env["gallery"] << "/" << ini[""]["thumb"] << "/"; // << file << ".jpg";

		do{
			DIR *dir = opendir(oldpath);
			if(!dir) break;

			CSString oldfile;
			int len = strlen(file);
			for(struct dirent *ent = readdir(dir); ent; ent = readdir(dir)){
				if(strncmp(file, ent->d_name, len))
					continue;

				oldfile  = oldpath;
				oldfile << ent->d_name;
				message << "Remove <tt>" << oldfile << "</tt><br>\n";
				unlink(oldfile);
			}
			closedir(dir);
		}while(0);
	}

	return 0;
}

int Run(void)
{
	CFormObj *obj;
	CSString hash, file;
	char *ptr;
	CSString path;

	int move = 1;
	if(!strlen(form["parent"]))
		move = 0;

	do{
		if(move) message << "move to " << form["parent"] << "<br>\n";
		else     message << "remove<br>\n";

		for(obj = form.First(); obj; obj = obj->Next()){
			if(!strcmp("move", obj->Name())){
				// get hash and filename
				hash = obj->Value();
				ptr  = strchr(hash, ' ');
				if(!ptr){
					message << sysmsg[PLUGIN_NAME]["data error"] << "<br>\n";
					message << obj->Value() << "<br>\n";
					continue;
				}
				*ptr++ = '\0';
				file = ptr;
				path  = env["home"];
				path << env["gallery"] << "/" << file;

				// test file
				if(access(path, W_OK)){
					message << sysmsg[PLUGIN_NAME]["file error"] << "<br>\n";
					message << "<tt>" << file << "</tt><br>\n";
					continue;
				}

				// (re)move file
				if(move) Move(file, form["parent"], hash);
				else	 Remove(file, hash);
			}
		}// for files
	}while(0);

	return 0;
}

int leaf(CDirTree &tree, CSString &node)
{
	static int loop = 0;
	if(++loop > 100)
		return 0;

	CSString nextnode, nextnode_t;
	if(node.Length()){
		nextnode_t = node;
		nextnode_t += '/';
	}

	CSString buf;
	for(int i = 0; tree[node]["entries"][i]; ++i){
		nextnode  = nextnode_t;
		nextnode += tree[node]["entries"][i];

		if(!*(tree[nextnode]["name"]))
			tree[nextnode]["name"] = tree[node]["entries"][i];
		printf("<option value=\"%s\">%s(%s)</option>\n", (string)nextnode, (string)tree[nextnode]["name"], (string)nextnode);

		if(tree[nextnode]["entries"][0]){
			leaf(tree, nextnode);
		}
	}

	return 0;
}

int ImageView(CImageList *imagelist)
{
	CImageList &images = *imagelist;
	CSString title;
	if(strlen((string)images[""]["title"]))
		title = (string)images[""]["title"];
	else if(strlen((string)tree[(string)env["gallery"]]["name"]))
		title = (string)tree[(string)env["gallery"]]["name"];
	else
		title = (string)env["gallery"];
	printf("<h4>%s(%s)</h4>\n", (string)title, (string)env["gallery"]);

	printf("<form action=\"%sgallery/%s%s\">"
		"<input type=\"hidden\" name=\"pv\" value=\""PLUGIN_NAME"\">"
		"<input type=\"hidden\" name=\"plugin\" value=\""PLUGIN_NAME"\">"
		"<input type=\"hidden\" name=\"action\" value=\"move\">"
		, (string)env["album"], (string)env["vhome"], (string)env["gallery"]);

	puts("<div id=\"images\">");

//	printf("pv="PLUGIN_NAME"<br>\n");

	int height = atoi(ini["thumbnail"]["height"]);
	if(height <= 0) height = 120;
	height += 40;

	CSString path, thumb, str;
	for(CImage *pImg = images.Group()->Next(); pImg; pImg = pImg->Next()){
		path  = MakeImageURI(pImg);
		thumb = MakeThumbnailURI(pImg);

	str.Format("<div align=\"center\" style=\"float: left; min-height: %d\"><a href=\"%s\" target=\"" PLUGIN_NAME "\"><img src=\"%s\"></a><br>"
			"<input type=\"checkbox\" name=\"move\" value=\"%s %s\">%s</div>"
			, height, (string)path, (string)thumb, (string)(*pImg)["hash"], pImg->Name(), pImg->Name());
		puts(str);
	}

	puts("</div><br style=\"clear: both;\">"); // images

	puts(sysmsg[PLUGIN_NAME]["parent"]);
	puts("<select name=\"parent\">");
	CSString dummy;
	leaf(tree, dummy);

	if(ini[PLUGIN_NAME]["remove"].IsTrue()){
		printf("<option value=\"\">%s</option>\n", (string)sysmsg[PLUGIN_NAME]["remove"]);
	}

	puts("</select> ");
	printf("<input type=\"submit\" value=\"%s\">", (string)sysmsg[PLUGIN_NAME]["submit"]);

	puts("</form>");

	puts("<div align=\"right\"><p>Copyright (C) 2006 Serika-Shiki Move Image Files Plugin</p></div>");
	return 0;
}

#if defined (__cplusplus)
}
#endif
