// sstring.cpp
////////////////////////////////////////

/*
 * NX (CSString)
 *   Copyright (C) 2006 肩
 *
 * MFCCString^Ƃقړł
 */

#include <stdio.h>
#include <stdarg.h>
#include <ctype.h>
#include <memory.h>

#include "sstring.h"

////////////////////////////////////////
// \zƏ
////////////////////////////////////////
CSString::CSString()
{
	m_pString    = NULL;
	m_Length     = 0;
	m_BufferSize = 0;
}

CSString::CSString(const CSString &str)
{
	m_Length = str.m_Length;
	m_BufferSize = str.m_BufferSize;
	m_pString = new char [m_BufferSize];
	if(!m_pString)
		throw _SSTRING_NOMEM;
	memcpy(m_pString, str.m_pString, m_BufferSize);
}

CSString::CSString(const char * const str)
{
	if(str){
		m_Length = strlen(str);
		m_BufferSize = alignment(m_Length);
		m_pString = new char [m_BufferSize];
		if(!m_pString)
			throw _SSTRING_NOMEM;
		memset(m_pString, 0, m_BufferSize);
		memcpy(m_pString, str, m_Length);
	}
}

CSString::CSString(const char ch)
{
	if(ch){
		m_Length = 1;
		m_BufferSize = alignment(m_Length);
		m_pString = new char [m_BufferSize];
		if(!m_pString)
			throw _SSTRING_NOMEM;
		memset(m_pString, 0, m_BufferSize);
		m_pString[0] = ch;
	}
}

CSString::~CSString()
{
	if(m_pString)  delete [] m_pString;
}

int CSString::Measure(void)
{
	if(m_pString)
		m_Length = strlen(m_pString);
	return m_Length;
}

void CSString::Empty(void)
{
	if(m_pString)  delete [] m_pString;
	m_BufferSize = 0;
	m_Length     = 0;
	m_pString    = NULL;
}

////////////////////////////////////////
// ̏
////////////////////////////////////////
// 
CSString & CSString::operator = (const CSString &str)
{
	if(str.IsEmpty()){
		Empty();
		return *this;
	}

	if(m_BufferSize <= str.m_Length){
		if(m_pString) delete [] m_pString;
		m_BufferSize = str.m_BufferSize;
		m_pString    = new char [m_BufferSize];
		if(!m_pString)
			throw _SSTRING_NOMEM;
	}

	m_Length = str.m_Length;
	strcpy(m_pString, str.m_pString);

	return *this;
}

CSString & CSString::operator = (const char * const str)
{
	if(!str || !*str){
		Empty();
		return *this;
	}

	if(m_BufferSize <= strlen(str)){
		if(m_pString) delete [] m_pString;
		m_BufferSize = alignment(strlen(str));
		m_pString    = new char [m_BufferSize];
		if(!m_pString)
			throw _SSTRING_NOMEM;
	}

	m_Length = strlen(str);
	strcpy(m_pString, str);

	return *this;
}

CSString & CSString::operator = (const char ch)
{
	if(!ch){
		Empty();
		return *this;
	}

	if(m_BufferSize <= 1){
		if(m_pString) delete [] m_pString;
		m_BufferSize = alignment(2);
		m_pString    = new char [m_BufferSize];
		if(!m_pString)
			throw _SSTRING_NOMEM;
	}

	m_Length = 1;
	m_pString[0] = ch;
	m_pString[1] = '\0';

	return *this;
}

// ǉ
CSString & CSString::operator += (const CSString &str)
{
	if(str.IsEmpty())
		return *this;

	if(this->IsEmpty())
		return (*this = str);

	if(m_BufferSize <= m_Length + str.m_Length){
		m_BufferSize = alignment(m_Length + str.m_Length);
		char *ptr = new char [m_BufferSize];
		if(!ptr)
			throw _SSTRING_NOMEM;
		if(m_pString){
			strcpy(ptr, m_pString);
			delete [] m_pString;
		}
		m_pString = ptr;
	}

	strcat(m_pString, str.m_pString);
	m_Length += str.m_Length;
	return *this;
}

CSString & CSString::operator += (const char * const str)
{
	if(!str || strlen(str) <= 0)
		return *this;

	if(this->IsEmpty())
		return (*this = str);

	if(m_BufferSize <= m_Length + strlen(str)){
		m_BufferSize = alignment(m_Length + strlen(str));
		char *ptr = new char [m_BufferSize];
		if(!ptr)
			throw _SSTRING_NOMEM;
		if(m_pString){
			strcpy(ptr, m_pString);
			delete [] m_pString;
		}
		m_pString = ptr;
	}

	strcat(m_pString, str);
	m_Length += strlen(str);
	return *this;
}

CSString & CSString::operator += (const char ch)
{
	if(ch == '\0')
		return *this;

	if(this->IsEmpty())
		return (*this = ch);

	if(m_BufferSize <= m_Length + 1){
		m_BufferSize = alignment(m_Length + 1);
		char *ptr = new char [m_BufferSize];
		if(!ptr)
			throw _SSTRING_NOMEM;
		if(m_pString){
			strcpy(ptr, m_pString);
			delete [] m_pString;
		}
		m_pString = ptr;
	}

	m_pString[  m_Length] = ch;
	m_pString[++m_Length] = '\0';
	return *this;
}

// 
CSString CSString::operator + (const CSString &str)
{
	if(str.IsEmpty())
		return *this;

	CSString obj(*this);
	obj += str;
	return obj;
}

CSString CSString::operator + (const char * const str)
{
	if(!str || !*str)
		return *this;

	CSString obj(*this);
	obj += str;
	return obj;
}

CSString CSString::operator + (const char ch)
{
	if(!ch)
		return *this;

	CSString obj(*this);
	obj += ch;
	return obj;
}

/*
CSString operator + (const char * const str1, CSString &str2)
{
	CSString obj(str1);
	obj += str2;
	return obj;
}

CSString operator + (const char ch, CSString &str2)
{
	CSString obj(ch);
	obj += str2;
	return obj;
}
*/

// t
CSString & CSString::Format(const char *format, ...)
{
	va_list ap;
//	FormatV(format, ap);

	int len;
	if(!m_BufferSize){
		len = strlen(format) + BUFFER_SIZE;
		m_BufferSize = alignment(len);
		m_pString = new char [m_BufferSize];
		if(!m_pString)
			throw _SSTRING_NOMEM;
	}

	do{
		va_start(ap, format);
		len = vsnprintf(m_pString, m_BufferSize, format, ap);
		va_end(ap);

		if(-1 < len && len < (signed long)m_BufferSize){
			m_Length = len;
			break;
		}
		if(-1 < len){
			m_BufferSize = alignment(len);
		}else{
			m_BufferSize += BUFFER_SIZE;
		}
		delete m_pString;
		m_pString = new char [m_BufferSize];
		if(!m_pString)
			throw _SSTRING_NOMEM;
	}while(1);

	return *this;
}

CSString & CSString::FormatV(const char *format, va_list ap)
{
	int len;
	if(!m_BufferSize){
		len = strlen(format) + BUFFER_SIZE;
		m_BufferSize = alignment(len);
		m_pString = new char [m_BufferSize];
		if(!m_pString)
			throw _SSTRING_NOMEM;
	}

	do{
		len = vsnprintf(m_pString, m_BufferSize, format, ap);
		if(-1 < len && len < (signed long)m_BufferSize){
			m_Length = len;
			break;
		}
		if(-1 < len){
			m_BufferSize = alignment(len);
		}else{
			m_BufferSize += BUFFER_SIZE;
		}
		delete m_pString;
		m_pString = new char [m_BufferSize];
		if(!m_pString)
			throw _SSTRING_NOMEM;
	}while(1);

	return *this;
}
/*
CSString & CSString::Printf(const char *format, ...)
{
	va_list ap;
	va_start(ap, format);
	FormatV(format, ap);
	va_end(ap);
	return *this;
}

CSString & CSString::VPrintf(const char *format, va_list ap)
{	return FormatV(format, ap); }
*/

// ̑Ő󔒗ޕ폜
CSString &CSString::Triming(void)
{
	if(this->IsEmpty())
		return *this;

	char *ptr;
	unsigned int i = 0;
	ptr = m_pString;
	while(*ptr && isspace(*ptr) && i < m_Length)
		++i, ++ptr;
	if(!*ptr){
		Empty();
		return *this;
	}

	CSString tmp(ptr);
	char *last = tmp.m_pString;
	ptr = tmp.m_pString;
	for(i = 0; i < m_Length && *ptr; ++i, ++ptr){
		if(!isspace(*ptr))
			last = ptr;
	}
	*++last = '\0';
	tmp.Measure();
	*this = tmp;

	return *this;
}

// Ō̈ꕶ폜
int CSString::Chop(void)
{
	if(this->IsEmpty())
		return -1;

	int old = -1;
	if(m_pString && m_Length > 0){
		old = m_pString[--m_Length];
		m_pString[m_Length] = '\0';
	}
	return old;
}

////////////////////////////////////////
// obt@ւ̃ANZX
////////////////////////////////////////
// ̎o
//char & CSString::GetAt(long index)
char CSString::GetAt(long index)
{
/*
	if(index > m_Length){
		throw _SSTRING_RANGE;
	}
	return m_pString[index];
*/

	if((unsigned long)index >= m_BufferSize){
		m_BufferSize = alignment(index);
		char *ptr = new char [m_BufferSize];
		if(!ptr)
			throw _SSTRING_NOMEM;
		if(m_pString){
			strcpy(ptr, m_pString);
			delete [] m_pString;
		}
		m_pString = ptr;
	}
	if((unsigned long)index >= m_Length){
		memset(&m_pString[m_Length], 0, m_BufferSize - m_Length);
		m_Length = index+1;
	}
	return m_pString[index];
}

////////////////////////////////////////
// vf̔r
////////////////////////////////////////
int CSString::operator == (const CSString &str)
{
	if(m_Length != str.m_Length)
		return 0;
	if(!m_Length) // m_Length == 0 && str.m_Length == 0
		return 1;
	if(!m_pString || !str.m_pString)
		return 0;
	return strcmp(m_pString, str.m_pString) == 0;
}
int CSString::operator == (const char *str)
{	return (*this) == CSString(str); }
int CSString::operator == (const char ch)
{	return (*this) == CSString(ch); }
/*
int operator == (const char *str1, const CSString &str2)
{	return CSString(str1) == str2; }
int operator == (const char ch, const CSString &str)
{	return CSString(ch) == str; }
*/

int CSString::operator != (const CSString &str)
{	return !((*this) == str); }
int CSString::operator != (const char *str)
{	return !((*this) == CSString(str)); }
int CSString::operator != (const char ch)
{	return !((*this) == CSString(ch)); }
/*
int operator != (const char *str1, const CSString &str2)
{	return !(CSString(str1) == str2); }
int operator != (const char ch, const CSString &str)
{	return !(CSString(ch) == str); }
*/

int CSString::operator > (const CSString &str)
{
	if(m_Length == 0)
		return 0;
	else if(str.m_Length == 0)
		return 1;
	return strcmp(m_pString, str.m_pString) > 0 ? 1 : 0;
}
int CSString::operator > (const char *str)
{	return ((*this) > CSString(str)); }
int CSString::operator > (const char ch)
{	return ((*this) > CSString(ch)); }

int CSString::operator >= (const CSString &str)
{
	if((*this) > str || (*this) == str)
		return 1;
	return 0;
}
int CSString::operator >= (const char *str)
{	return ((*this) >= CSString(str)); }
int CSString::operator >= (const char ch)
{	return ((*this) >= CSString(ch)); }

int CSString::operator < (const CSString &str)
{	return !((*this) >= str); }
int CSString::operator < (const char *str)
{	return !((*this) >= CSString(str)); }
int CSString::operator < (const char ch)
{	return !((*this) >= CSString(ch)); }

int CSString::operator <= (const CSString &str)
{	return !((*this) >  str); }
int CSString::operator <= (const char *str)
{	return !((*this) >  CSString(str)); }
int CSString::operator <= (const char ch)
{	return !((*this) >  CSString(ch)); }

int CSString::Compare(const CSString &str)
{
	if(m_BufferSize == 0 && str.m_BufferSize == 0)
		return 0;
	else if(m_BufferSize == 0)
		return 1;
	else if(str.m_BufferSize == 0)
		return -1;
	return strcmp(m_pString, str.m_pString);
}
int CSString::Compare(const char *str)
{	return Compare(CSString(str)); }
int CSString::Compare(const char ch)
{	return Compare(CSString(ch)); }

////////////////////////////////////////
// ̎o
////////////////////////////////////////
// w肵̐
int CSString::Count(int ch) const
{
	int num = 0;
	if(!IsEmpty()){
		char *ptr = m_pString;
		unsigned int i;
		for(i = 0; i < m_Length; ++i)
			if(ptr[i] == ch)
				++num;
	}
	return num;
}

char *CSString::strchr(int ch, int count) const
{
	char *ptr = NULL;
	if(!IsEmpty() && count >= 0){
		if(count == 0)
			return m_pString;
		ptr = ::strchr(m_pString, ch);
		while(--count > 0 && ptr && *ptr)
			ptr = ::strchr(++ptr, ch);
	}
	return ptr;
}

char *CSString::strrchr(int ch) const
{
	char *ptr = NULL;
	if(!IsEmpty()){
		ptr = ::strrchr(m_pString, ch);
	}
	return ptr;
}
